Package: IMMAN
Title: Interlog protein network reconstruction by Mapping and Mining
        ANalysis
Version: 1.32.0
Description: Reconstructing Interlog Protein Network (IPN) integrated from several 
             Protein protein Interaction Networks (PPINs). Using this package, 
             overlaying different PPINs to mine conserved common networks between diverse 
             species will be applicable.
Author: Minoo Ashtiani,
        Payman Nickchi, 
        Abdollah Safari,
        Mehdi Mirzaie,
        Mohieddin Jafari
Maintainer: Minoo Ashtiani <ashtiani.minoo@gmail.com>
biocViews: SequenceMatching, Alignment, SystemsBiology,
        GraphAndNetwork, Network, Proteomics
RoxygenNote: 7.3.1
License: Artistic-2.0
Encoding: UTF-8
LazyData: true
Imports: STRINGdb, pwalign, igraph, graphics, utils, seqinr
Suggests: knitr, rmarkdown, testthat
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/IMMAN
git_branch: RELEASE_3_23
git_last_commit: ca9cbb8
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:44:47 UTC; biocbuild
