# Generated by roxygen2: do not edit by hand

export(IMMAN)
importFrom(STRINGdb,STRINGdb)
importFrom(graphics,plot)
importFrom(igraph,graph_from_data_frame)
importFrom(igraph,layout_in_circle)
importFrom(pwalign,pairwiseAlignment)
importFrom(pwalign,pid)
importFrom(seqinr,read.fasta)
importFrom(utils,read.csv)
