#' @title Fruit Fly
#'
#' @description A sample collection of 56 UNIPROT_AC of Fruit Fly species.
#' @name FruitFly
#' @docType data
#' @usage data("FruitFly")
#' @format a character vector
#' @references http://www.uniprot.org/
#' @keywords datasets
#' @examples
#' data("FruitFly")
#' print(FruitFly)
NULL
