#' @title Homo sapiens (Human)
#'
#' @description A sample collection of 76 UNIPROT_AC of Homo sapiens species.
#' @name H.sapiens
#' @docType data
#' @usage data("H.sapiens")
#' @format a character vector
#' @references http://www.uniprot.org/
#' @keywords datasets
#' @examples
#' data("H.sapiens")
#' print(H.sapiens)
NULL
