#' @title Celegance
#'
#' @description A sample collection of 49 UNIPROT_AC of Celegance species.
#' @name Celegance
#' @docType data
#' @usage data("Celegance")
#' @format a character vector
#' @references http://www.uniprot.org/
#' @keywords datasets
#' @examples
#' data("Celegance")
#' print(Celegance)
NULL
