#' @title Rattus norvegicus (Rat)
#'
#' @description A sample collection of 70 UNIPROT_AC of Rattus norvegicus species.
#' @name R.norvegicus
#' @docType data
#' @usage data("R.norvegicus")
#' @format a character vector
#' @references http://www.uniprot.org/
#' @keywords datasets
#' @examples
#' data("R.norvegicus")
#' print(R.norvegicus)
NULL
